# Author: Oriol Filter
# Date: 06/04/2022


$DebugPreference = "Continue"
$ErrorActionPreference = "Stop"
$SCRIPT_PATH=$PSScriptRoot
$HYPERV_FILE = "$SCRIPT_PATH/hyperv_list.txt"
$SCVMM_FILE = "$SCRIPT_PATH/hyperv_list.txt"
$HOSTS_FILE = "$SCRIPT_PATH/hosts.txt"
## Objects


## Add Vhost objects for each type of server

class VirtualizationServer {
    [int]$port=8100
    [string]$url
#     [Boolean] _FindHost ($vname) {
#         $type=$this.GetType().BaseType
#         Write-Host "Find Host not implemented in type $type"
#         return $false
#     }
#     [Boolean] FindHost ($vname) {
#         return $this._FindHost($vname)
#     }
    [Hashtable]$__hostlist=@{}

    __load_hosts(){
        $type=$this.GetType().BaseType
        Write-Host "Load hosts not implemented in type $type"
    }

    load_hosts(){
        $this.__host_list=@()
        $this.__load_hosts()
    }

    hidden [Array] $__host_list=@()
    [Array] $host_list = $this.__host_list

}
class SystemCenterVirtualMachineManager {
    __load_hosts(){
        Get-SCVMMServer -ComputerName $this.url -TCPPort $this.port
        $list = Get-SCVirtualMachine
        $this.__host_list=$list
#         foreach ($element in $list){$element.Name}
        Format("Loaded VM from {0}",$this.url) | Write-Debug
    }
}
class HypervisorServer: VirtualizationServer{
#     [Boolean] _FindHost ($vname) {
#         $result=$false
#         try
#         {
#             Invoke-Command -ComputerName $this.url {
#                 Param ($name) Get-VM -Name $name
#             } -ArgumentList $vname
#             $result=$?
#         }
#         catch
#         {
#             Format ("{0}: {1}",$this.url,$Error[0]) | Write-Debug
#         }
#
#         return $result
#     }
}
class VMWareServer: VirtualizationServer{}

class VirtualizerManager{
    [Array]$__connected_servers=@()
    [Hashtable]$__found_vh=@{}
    [Boolean] __log_in($url){
        $status_message=""
        $result=$false
        try {
            $result=Invoke-Command -ComputerName $url { $true }
            $status_message = "OK"
        }
        catch [System.Management.Automation.Remoting.PSRemotingTransportException] {
            $status_message = Format("Error '{0}'",$_.Exception.GetType().fullname)
        }
        finally {
            $message = [string]::Format(">> [{0}] {1}",$url,$status_message)
            Write-Debug $message
        }
        return $result
    }
    [Hashtable] $found_vh = $this.__found_vh

    append_session($type,$url)
    {
        $type.GetType().IsValueType
        $result = $this.__log_in($url)
        if ($result)
        {
            $server=$type::new()
            $server.url=$url
            $this.__connected_servers+=$server
        }
    }
    [Array] connected_servers() {
        return $this.__connected_servers
    }

    load_hosts() {
        $this.__found_vh=@{}
        for ($num = 0 ; $num -le $this.__connected_servers.Length -1 ; $num++){
               $server = $this.__connected_servers[$num]
               $server.load_hosts()
               foreach ($item in $server.host_list)
               {
                    $this.append_vhost
               }
        }
    }
    __append_vhost($vhost,$source_url){
        if ($this.__host_list[$vhost.Name] -isnot [Hashtable]){
            $this.__host_list[$vhost.Name]=@{}
        }
        $this.__host_list[$vhost.Name.ToLower()][$source_url]=$vhost
    }
    append_vhost($vhost,$source_url){
        $this.__append_vhost($vhost,$source_url)
    }
    find_host($vname){
        $hostlist=$vname
        if ($hostlist -is [Array]){
            $hostlist=@($hostlist)
        }
        # Find hosts
        foreach ($vhostname  in $hostlist) {
            $item_from_list=$this.found_vh[$vhostname.ToString()]
            [String]::Format("`t{0}:",$vhostname.name) | Write-Host
            if (.Length > 0)
            {
                for ($num = 0 ; $num -le $item_from_list.Length -1 ; $num++){
                    $server = $item_from_list[$num]
                    [String]::Format("`t{0}:",$server.url) | Write-Host
                }
            }
            else {
                [String]::Format("`t---- Couldn't be found") | Write-Host
            }
        }
    }
#     find_host($vname){
#         $hostlist=$vname
#         if ($hostlist -is [Array]){
#             $hostlist=@($hostlist)
#         }
#         foreach ($_vname in $hostlist)
#         {
#             for ($num = 0 ; $num -le $this.__connected_servers.Length -1 ; $num++){
#                     $server= $this.__connected_servers[$num]
#                     $result = $server.FindHost($_vname)
#                     [String]::Format(">> {0} found in {1}? {2}", $_vname, $server.url, $result) | Write-Host
#             }
#             }
#         }
}

function Get-Hosts-From-File($filepath)
{
    ## In a future use a yaml as a .conf (with the hyperv and vmware listed)
    $file_content = Get-Content "$filepath"
    $splitted_file_content = $file_content.Split()
    $host_array=@()
    foreach ($hostname in $splitted_file_content)
    {
            if ($hostname)
        {
            $host_array+=$hostname
        }
    }
    return $host_array
}


# Advanced params?
# https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_functions_advanced_parameters?view=powershell-7.2
function load_hyperv($manager) {
    $hyperv_url = Get-Hosts-From-File($HYPERV_FILE)
    foreach ($url in $hyperv_url) {
        $manager.append_session([HypervisorServer], $url)
    }
}

function load_scvmm($manager) {
    $hyperv_url = Get-Hosts-From-File($SCVMM_FILE)
    foreach ($url in $hyperv_url) {
        $manager.append_session([SystemCenterVirtualMachineManager], $url)
    }
}

function Main{
    $manager = [VirtualizerManager]::new()
    load_scvmm($manager)
    $manager.__connected_servers
#    $vmhosts = Get-Hosts-From-File($HOSTS_FILE)
#    $manager.find_host($vmhosts)
}


Main
